@@ -16,6 +16,7 @@
 #endif
 
 #include <string>
+#include <memory>
 #include <vector>
 
 #include "nlohmann/json.hpp"
@@ -35,10 +36,12 @@ namespace xpyt
     {
     public:
 
+        using gil_scoped_release_ptr = std::unique_ptr<py::gil_scoped_release>;
+
         interpreter(int argc, const char* const* argv);
         virtual ~interpreter();
 
-    private:
+    protected:
 
         void configure_impl() override;
 
@@ -61,11 +64,20 @@ namespace xpyt
 
         void shutdown_request_impl() override;
 
+        nl::json internal_request_impl(const nl::json& content) override;
+
         void redirect_output();
         void redirect_display();
 
         py::object m_displayhook;
         std::vector<std::string> m_inputs;
+
+        // The interpreter has the same scope as a `gil_scoped_release` instance
+        // so that the GIL is not held by default, it will only be held when the
+        // interpreter wants to execute Python code. This means that whenever
+        // the interpreter will execute Python code it will need to create an
+        // `gil_scoped_acquire` instance first.
+        gil_scoped_release_ptr m_release_gil = nullptr;
     };
 }
 
